from unittest import TestCase
from . import labelling_tool


class LabelClassTestCase(TestCase):
    def test_constructor_simple(self):
        metal = labelling_tool.LabelClass('metal', 'Metal')
        self.assertEqual(metal.name, 'metal')
        self.assertEqual(metal.human_name, 'Metal')
        self.assertIsNone(metal.colours)

    def test_constructor_colour(self):
        metal = labelling_tool.LabelClass('metal', 'Metal', colour=(1, 2, 3))
        self.assertEqual(metal.name, 'metal')
        self.assertEqual(metal.human_name, 'Metal')
        self.assertEqual(metal.colours, {'default': [1, 2, 3]})

        metal = labelling_tool.LabelClass('metal', 'Metal', colour=[1, 2, 3])
        self.assertEqual(metal.name, 'metal')
        self.assertEqual(metal.human_name, 'Metal')
        self.assertEqual(metal.colours, {'default': [1, 2, 3]})

    def test_constructor_colours(self):
        metal = labelling_tool.LabelClass('metal', 'Metal', colours={'basic': (1, 2, 3), 'more': [2, 3, 4]})
        self.assertEqual(metal.name, 'metal')
        self.assertEqual(metal.human_name, 'Metal')
        self.assertEqual(metal.colours, {'basic': [1, 2, 3], 'more': [2, 3, 4]})

    def test_constructor_colours_bad(self):
        self.assertRaises(TypeError,
                          lambda: labelling_tool.LabelClass('metal', 'Metal', colour=(1, 2, 3),
                                                            colours={'basic': (1, 2, 3), 'more': (2, 3, 4)}))

    def test_to_json(self):
        metal_a = labelling_tool.LabelClass('metal', 'Metal')
        self.assertEqual(metal_a.to_json(), dict(name='metal', human_name='Metal', colours=None))
        metal_b = labelling_tool.LabelClass('metal', 'Metal', colours={'basic': (1, 2, 3), 'more': [2, 3, 4]})
        self.assertEqual(metal_b.to_json(), dict(name='metal', human_name='Metal',
                                                 colours={'basic': [1, 2, 3], 'more': [2, 3, 4]}))


class LabelClassGroupTestCase(TestCase):
    def test_constructor(self):
        metal = labelling_tool.LabelClass('metal', 'Metal')
        wood = labelling_tool.LabelClass('wood', 'Wood')
        group = labelling_tool.LabelClassGroup('Materials', [metal, wood])
        self.assertEqual(group.group_name, 'Materials')
        self.assertEqual(group.classes, [metal, wood])

    def test_to_json(self):
        metal = labelling_tool.LabelClass('metal', 'Metal')
        wood = labelling_tool.LabelClass('wood', 'Wood')
        group = labelling_tool.LabelClassGroup('Materials', [metal, wood])
        self.assertEqual(group.to_json(),
            dict(group_name='Materials', group_classes=[dict(name='metal', human_name='Metal', colours=None),
                                                        dict(name='wood', human_name='Wood', colours=None)]))



# The MIT License (MIT)
#
# Copyright (c) 2015 University of East Anglia, Norwich, UK
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# Developed by Geoffrey French in collaboration with Dr. M. Fisher and
# Dr. M. Mackiewicz.
from PyQt5 import QtWidgets, QtWebEngineWidgets
import click
from image_labelling_tool_qt import controls, web_server


@click.command(context_settings=dict(ignore_unknown_options=True, allow_extra_args=True))
@click.option('--dextr_weights', type=click.Path())
@click.option('--enable_firebug', is_flag=True, default=False)
@click.option('--use_http_memory_cache', is_flag=True, default=False)
def run_app(dextr_weights, enable_firebug, use_http_memory_cache):
    import pathlib
    import glob
    from image_labelling_tool import labelled_image, labelling_tool

    # Create an application
    app = QtWidgets.QApplication([])

    # Start with a dialog that allows the user to choose the images directory, optionally
    # a labels directory and read only checkbox
    init_dialog = QtWidgets.QDialog()
    # Dialog layout
    dia_layout = QtWidgets.QGridLayout()
    init_dialog.setLayout(dia_layout)

    images_dir = ['.']
    labels_dir = [None]

    # Prompt
    prompt_label = QtWidgets.QLabel('Choose an images directory and optionally a labels directory.')
    dia_layout.addWidget(prompt_label, 0, 0, 1, 2)

    # Images directory
    images_dir_button = QtWidgets.QPushButton('Choose images directory')
    images_dir_button.setIcon(init_dialog.style().standardIcon(QtWidgets.QStyle.SP_DirOpenIcon))
    images_dir_label = QtWidgets.QLabel(images_dir[0])

    def _on_images_dir_button():
        file_dialog = QtWidgets.QFileDialog(None, 'Choose images directory', '')
        file_dialog.setFileMode(QtWidgets.QFileDialog.Directory)
        choice = file_dialog.exec()
        if choice == QtWidgets.QFileDialog.Accepted:
            images_dir[0] = file_dialog.selectedFiles()[0]
            images_dir_label.setText(images_dir[0])

    images_dir_button.clicked.connect(_on_images_dir_button)
    dia_layout.addWidget(images_dir_button, 1, 0)
    dia_layout.addWidget(images_dir_label, 1, 1)

    # Labels directory
    labels_dir_button = QtWidgets.QPushButton('[Optional] Choose labels directory')
    labels_dir_button.setIcon(init_dialog.style().standardIcon(QtWidgets.QStyle.SP_DirOpenIcon))
    labels_dir_label = QtWidgets.QLabel('')

    def _on_labels_dir_button():
        file_dialog = QtWidgets.QFileDialog(None, 'Choose labels directory', '')
        file_dialog.setFileMode(QtWidgets.QFileDialog.Directory)
        choice = file_dialog.exec()
        if choice == QtWidgets.QFileDialog.Accepted:
            labels_dir[0] = file_dialog.selectedFiles()[0]
            labels_dir_label.setText(labels_dir[0])

    labels_dir_button.clicked.connect(_on_labels_dir_button)
    dia_layout.addWidget(labels_dir_button, 2, 0)
    dia_layout.addWidget(labels_dir_label, 2, 1)

    # Read only checkbox
    read_only_check = QtWidgets.QCheckBox('Read only')
    dia_layout.addWidget(read_only_check, 3, 1)

    # Read only checkbox
    read_only_check = QtWidgets.QCheckBox('Read only')
    dia_layout.addWidget(read_only_check, 3, 1)

    # DEXTR checkbox
    dextr_check = QtWidgets.QCheckBox('Enable DEXTR')
    dia_layout.addWidget(dextr_check, 4, 1)

    # Buttons
    ok_button = QtWidgets.QPushButton('Ok')
    ok_button.setIcon(init_dialog.style().standardIcon(QtWidgets.QStyle.SP_DialogOkButton))

    def _on_ok():
        init_dialog.accept()

    ok_button.clicked.connect(_on_ok)
    cancel_button = QtWidgets.QPushButton('Cancel')
    cancel_button.setIcon(init_dialog.style().standardIcon(QtWidgets.QStyle.SP_DialogCancelButton))

    def _on_cancel():
        init_dialog.reject()

    cancel_button.clicked.connect(_on_cancel)
    dia_layout.addWidget(ok_button, 5, 0)
    dia_layout.addWidget(cancel_button, 5, 1)

    # Run the dialog
    action = init_dialog.exec()

    if action == QtWidgets.QDialog.Accepted:
        images_dir = pathlib.Path(images_dir[0])
        if labels_dir[0] is not None:
            labels_dir = pathlib.Path(labels_dir[0])
        else:
            labels_dir = None
        enable_dextr = dextr_check.checkState()
        readonly = read_only_check.checkState()

        server = web_server.LabellerServer()
        server.start_flask_server()

        try:
            # If DEXTR is to be made available
            if enable_dextr or dextr_weights is not None:
                from dextr.model import DextrModel
                import torch

                # Load the dextr model
                device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")

                if dextr_weights is not None:
                    dextr_weights = pathlib.Path(dextr_weights).expanduser()
                    dextr_model = torch.load(dextr_weights, map_location=device)
                else:
                    dextr_model = DextrModel.pascalvoc_resunet101().to(device)

                # Evaluation mode
                dextr_model.eval()

                # Define a mask prediction function
                dextr_fn = lambda image, points: dextr_model.predict([image], points[None, :, :])[0] >= 0.5
            else:
                dextr_fn = None


            # Colour schemes
            # The user may select different colour schemes for different tasks.
            # If you have a lot of classes, it will be difficult to select colours that are easily distinguished
            # from one another. For one task e.g. segmentation, design a colour scheme that highlights the different
            # classes for that task, while another task e.g. fine-grained classification would use another scheme.
            # Each colour scheme is a dictionary containing the following:
            #   name: symbolic name (Python identifier)
            #   human_name: human readable name for UI
            # These colour schemes are going to split the classes by 'default' (all), natural, and artificial.
            # Not really useful, but demonstrates the feature.
            colour_schemes = [
                dict(name='default', human_name='All'),
                dict(name='natural', human_name='Natural'),
                dict(name='artificial', human_name='Artifical')
            ]

            # Specify our label classes, organised in groups.
            # `LabelClass` parameters are:
            #   symbolic name (Python identifier)
            #   human readable name for UI
            #   and colours by colour scheme, as a dict mapping colour scheme name to RGB value as a list
            # The label classes are arranged in groups and will be displayed as such in the UI.
            # `LabelClassGroup` parameters are:
            #   human readable name for UI
            #   label class (`LabelClass` instance) list
            label_classes = [
                labelling_tool.LabelClassGroup('Natural', [
                    labelling_tool.LabelClass('tree', 'Trees', dict(default=[0, 255, 192], natural=[0, 255, 192],
                                                                    artificial=[128, 128, 128])),
                    labelling_tool.LabelClass('lake', 'Lake', dict(default=[0, 128, 255], natural=[0, 128, 255],
                                                                   artificial=[128, 128, 128])),
                    labelling_tool.LabelClass('flower', 'Flower', dict(default=[255, 96, 192], natural=[255, 192, 96],
                                                                       artificial=[128, 128, 128])),
                    labelling_tool.LabelClass('leaf', 'Leaf', dict(default=[65, 255, 0], natural=[65, 255, 0],
                                                                   artificial=[128, 128, 128])),
                    labelling_tool.LabelClass('stem', 'Stem', dict(default=[128, 64, 0], natural=[128, 64, 0],
                                                                   artificial=[128, 128, 128])),
                ]),
                labelling_tool.LabelClassGroup('Artificial', [
                    labelling_tool.LabelClass('building', 'Buildings', dict(default=[255, 128, 0], natural=[128, 128, 128],
                                                                           artificial=[255, 128, 0])),
                    labelling_tool.LabelClass('wall', 'Wall', dict(default=[0, 128, 255], natural=[128, 128, 128],
                                                                   artificial=[0, 128, 255])),
                ])]

            # Annotation controls
            # Labels may also have optional meta-data associated with them
            # You could use this for e.g. indicating if an object is fully visible, mostly visible or significantly obscured.
            # You could also indicate quality (e.g. blurriness, etc)
            # There are four types of annotation. They have some common properties:
            #   - name: symbolic name (Python identifier)
            #   - label_text: label text in UI
            #   Check boxes, radio buttons and popup menus also have:
            #     - visibility_label_text: [optional] if provided, label visibility can be filtered by this annotation value,
            #       in which case a drop down will appear in the UI allowing the user to select a filter value
            #       that will hide/show labels accordingly
            # Control types:
            # Check box (boolean value):
            #   `labelling_tool.AnnoControlCheckbox`; only the 3 common parameters listed above
            # Radio button (choice from a list):
            #   `labelling_tool.AnnoControlRadioButtons`; the 3 common parameters listed above and:
            #       choices: list of `labelling_tool.AnnoControlRadioButtons.choice` that provide:
            #           value: symbolic value name for choice
            #           tooltip: extra information for user
            #       label_on_own_line [optional]: if True, place the label and the buttons on a separate line in the UI
            # Popup menu (choice from a grouped list):
            #   `labelling_tool.AnnoControlPopupMenu`; the 3 common parameters listed above and:
            #       groups: list of groups `labelling_tool.AnnoControlPopupMenu.group`:
            #           label_text: label text in UI
            #           choices: list of `labelling_tool.AnnoControlPopupMenu.choice` that provide:
            #               value: symbolic value name for choice
            #               label_text: choice label text in UI
            #               tooltip: extra information for user
            # Text (free form plain text):
            #   `labelling_tool.AnnoControlText`; only the 2 common parameters listed above and:
            #       - multiline: boolean; if True a text area will be used, if False a single line text entry
            anno_controls = [
                labelling_tool.AnnoControlCheckbox('good_quality', 'Good quality',
                                                   visibility_label_text='Filter by good quality'),
                labelling_tool.AnnoControlRadioButtons('visibility', 'Visible', choices=[
                    labelling_tool.AnnoControlRadioButtons.choice(value='full', label_text='Fully',
                                                                  tooltip='Object is fully visible'),
                    labelling_tool.AnnoControlRadioButtons.choice(value='mostly', label_text='Mostly',
                                                                  tooltip='Object is mostly visible'),
                    labelling_tool.AnnoControlRadioButtons.choice(value='obscured', label_text='Obscured',
                                                                  tooltip='Object is significantly obscured'),
                ], label_on_own_line=False, visibility_label_text='Filter by visibility'),
                labelling_tool.AnnoControlPopupMenu('material', 'Material', groups=[
                    labelling_tool.AnnoControlPopupMenu.group(label_text='Artifical/buildings', choices=[
                        labelling_tool.AnnoControlPopupMenu.choice(value='concrete', label_text='Concrete',
                                                                   tooltip='Concrete objects'),
                        labelling_tool.AnnoControlPopupMenu.choice(value='plastic', label_text='Plastic',
                                                                   tooltip='Plastic objects'),
                        labelling_tool.AnnoControlPopupMenu.choice(value='asphalt', label_text='Asphalt',
                                                                   tooltip='Road, pavement, etc.'),
                    ]),
                    labelling_tool.AnnoControlPopupMenu.group(label_text='Flat natural', choices=[
                        labelling_tool.AnnoControlPopupMenu.choice(value='grass', label_text='Grass',
                                                                   tooltip='Grass covered ground'),
                        labelling_tool.AnnoControlPopupMenu.choice(value='water', label_text='Water',
                                                                   tooltip='Water/lake')]),
                    labelling_tool.AnnoControlPopupMenu.group(label_text='Vegetation', choices=[
                        labelling_tool.AnnoControlPopupMenu.choice(value='trees', label_text='Trees', tooltip='Trees'),
                        labelling_tool.AnnoControlPopupMenu.choice(value='shrubbery', label_text='Shrubs',
                                                                   tooltip='Shrubs/bushes'),
                        labelling_tool.AnnoControlPopupMenu.choice(value='flowers', label_text='Flowers',
                                                                   tooltip='Flowers'),
                        labelling_tool.AnnoControlPopupMenu.choice(value='ivy', label_text='Ivy', tooltip='Ivy')]),
                ], visibility_label_text='Filter by material'),
                # labelling_tool.AnnoControlText('comment', 'Comment', multiline=False),
            ]

            image_paths = list(images_dir.glob('*.jpg')) + list(images_dir.glob('*.png'))
            image_paths = [p.absolute() for p in image_paths]

            # Create a `PersistentLabelledImage` instance for each image file
            labelled_images = labelled_image.LabelledImage.for_image_files(
                image_paths, labels_dir=labels_dir, readonly=bool(readonly))
            print('Loaded {0} images'.format(len(labelled_images)))


            config = web_server.DEFAULT_CONFIG

            # Example tasks to appear in checkboxes
            tasks = [
                dict(name='finished', human_name='[old] finished'),
                dict(name='segmentation', human_name='Outlines'),
                dict(name='classification', human_name='Classification'),
            ]

            # And a window
            win = QtWidgets.QWidget()
            win.setWindowTitle('Simple Qt Image Labeller')

            # And give it a layout
            layout = QtWidgets.QVBoxLayout()
            win.setLayout(layout)

            # Create the labeller
            lbl = controls.QLabellerForLabelledImages(
                server=server, label_classes=label_classes, labelled_images=labelled_images,
                tasks=tasks, colour_schemes=colour_schemes,
                anno_controls=anno_controls, config=config, dextr_fn=dextr_fn,
                enable_firebug=enable_firebug)
            # Create the web engine view
            view = QtWebEngineWidgets.QWebEngineView()

            # If requested, use a memory-based HTTP cache
            # This is very useful if the client-side Javascript code is being developed
            # as otherwise chromium's cache will often store old versions of the code that
            # will hamper debugging and deveopment
            if use_memory_cache:
                page.profile().setHttpCacheType(QtWebEngineWidgets.QWebEngineProfile.MemoryHttpCache)

            # Attach the labeller to the web engine view
            lbl.attach_to_web_engine_view(view)

            # Add the QWebView to the layout
            layout.addWidget(view)

            # Show the window and run the app
            win.show()

            app.exec_()
        finally:
            print('Stopping server')
            server.stop_server()


if __name__ == '__main__':
    run_app()


import datetime
from django.db import models
from django.db.models import Q
from django.utils import timezone


class LabelsManager (models.Manager):
    def empty(self):
        return self.filter(labels_json_str='[]')

    def not_empty(self):
        return self.exclude(labels_json_str='[]')

    def modified_by_user(self, user):
        return self.filter(last_modified_by=user)

    def locked_by_user(self, user):
        return self.filter(locked_by=user)

    def unlocked(self):
        return self.filter(self.unlocked_q())

    @staticmethod
    def unlocked_q():
        now = timezone.now()
        return Q(locked_by=None) | Q(lock_expiry_datetime__gte=now)





import os, mimetypes, json, datetime, uuid
from django.core.management.base import BaseCommand, CommandError
from django.core.files import File
from django.db import transaction
from image_labelling_tool import labelling_tool
from image_labelling_tool import models
from ... import models

class Command(BaseCommand):
    help = 'Updates object IDs to use the new UUID based format in label JSON'

    def handle(self, *args, **options):
        # It should be impossible for uuid.uuid4 to generate duplicate UUIDs, but
        # keep track of the ones we've created just in case
        used_uuids = set()
        n_processed = 0
        n_updated = 0
        with transaction.atomic():
            for labels in models.Labels.objects.all():
                labels_js = json.loads(labels.labels_json_str)
                id_prefix = str(uuid.uuid4())
                while id_prefix in used_uuids:
                    id_prefix = str(uuid.uuid4())
                used_uuids.add(id_prefix)
                modified = labelling_tool.ensure_json_object_ids_have_prefix(
                    labels_js, id_prefix=id_prefix)
                if modified:
                    labels.labels_json_str = json.dumps(labels_js)
                    labels.save()
                    n_updated += 1
                n_processed += 1
        print('Updated {}/{} Label models'.format(n_updated, n_processed))



# The MIT License (MIT)
#
# Copyright (c) 2015 University of East Anglia, Norwich, UK
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# Developed by Geoffrey French in collaboration with Dr. M. Fisher and
# Dr. M. Mackiewicz.
import click


def flask_labeller(label_classes, labelled_images, tasks=None, colour_schemes=None, anno_controls=None,
                   config=None, dextr_fn=None, use_reloader=True, debug=True, port=None):
    import json
    import uuid
    import numpy as np

    from flask import Flask, render_template, request, make_response, send_file
    try:
        from flask_socketio import SocketIO, emit as socketio_emit
    except ImportError:
        SocketIO = None
        socketio_emit = None

    from image_labelling_tool import labelling_tool

    # Generate image IDs list
    image_ids = [str(i)   for i in range(len(labelled_images))]
    # Generate images table mapping image ID to image so we can get an image by ID
    images_table = {image_id: img   for image_id, img in zip(image_ids, labelled_images)}
    # Generate image descriptors list to hand over to the labelling tool
    # Each descriptor provides the image ID, the URL and the size
    image_descriptors = []
    for image_id, img in zip(image_ids, labelled_images):
        height, width = img.image_source.image_size
        image_descriptors.append(labelling_tool.image_descriptor(
            image_id=image_id, url='/image/{}'.format(image_id),
            width=width, height=height
        ))


    app = Flask(__name__, static_folder='static')
    if SocketIO is not None:
        print('Using web sockets')
        socketio = SocketIO(app)
    else:
        socketio = None


    def apply_dextr_js(image, dextr_points_js):
        image_for_dextr = image.image_source.image_as_array_or_pil()
        dextr_points = np.array([[p['y'], p['x']] for p in dextr_points_js])
        if dextr_fn is not None:
            mask = dextr_fn(image_for_dextr, dextr_points)
            regions = labelling_tool.PolygonLabel.mask_image_to_regions_cv(mask, sort_decreasing_area=True)
            regions_js = labelling_tool.PolygonLabel.regions_to_json(regions)
            return regions_js
        else:
            return []




    if config is None:
        config = labelling_tool.DEFAULT_CONFIG


    @app.route('/')
    def index():
        label_classes_json = [(cls.to_json() if isinstance(cls, labelling_tool.LabelClassGroup) else cls)
                               for cls in label_classes]
        if anno_controls is not None:
            anno_controls_json = [c.to_json() for c in anno_controls]
        else:
            anno_controls_json = []
        return render_template('labeller_page.jinja2',
                               tasks=tasks,
                               colour_schemes=colour_schemes,
                               label_class_groups=label_classes_json,
                               image_descriptors=image_descriptors,
                               initial_image_index=0,
                               anno_controls=anno_controls_json,
                               labelling_tool_config=config,
                               dextr_available=dextr_fn is not None,
                               use_websockets=socketio is not None)


    if socketio is not None:
        @socketio.on('get_labels')
        def handle_get_labels(arg_js):
            image_id = arg_js['image_id']

            image = images_table[image_id]

            wrapped_labels = image.labels_store.get_wrapped_labels()

            label_header = dict(image_id=image_id,
                                labels=wrapped_labels.labels_json,
                                completed_tasks=wrapped_labels.completed_tasks,
                                timeElapsed=0.0,
                                state='editable',
                                session_id=str(uuid.uuid4()),
            )

            socketio_emit('get_labels_reply', label_header)


        @socketio.on('set_labels')
        def handle_set_labels(arg_js):
            label_header = arg_js['label_header']

            image_id = label_header['image_id']

            image = images_table[image_id]

            wrapped_labels = image.labels_store.get_wrapped_labels()
            wrapped_labels.labels_json = label_header['labels']
            wrapped_labels.completed_tasks = label_header['completed_tasks']
            image.labels_store.update_wrapped_labels(wrapped_labels)

            socketio_emit('set_labels_reply', '')


        @socketio.on('dextr')
        def handle_dextr(dextr_js):
            if 'request' in dextr_js:
                dextr_request_js = dextr_js['request']
                image_id = dextr_request_js['image_id']
                dextr_id = dextr_request_js['dextr_id']
                dextr_points = dextr_request_js['dextr_points']

                image = images_table[image_id]

                regions_js = apply_dextr_js(image, dextr_points)

                dextr_labels = dict(image_id=image_id, dextr_id=dextr_id, regions=regions_js)
                dextr_reply = dict(labels=[dextr_labels])

                socketio_emit('dextr_reply', dextr_reply)
            elif 'poll' in dextr_js:
                dextr_reply = dict(labels=[])
                socketio_emit('dextr_reply', dextr_reply)
            else:
                dextr_reply = {'error': 'unknown_command'}
                socketio_emit('dextr_reply', dextr_reply)


    else:
        @app.route('/labelling/get_labels/<image_id>')
        def get_labels(image_id):
            image = images_table[image_id]
            wrapped_labels = image.labels_store.get_wrapped_labels()

            label_header = {
                'image_id': image_id,
                'labels': wrapped_labels.labels_json,
                'completed_tasks': wrapped_labels.completed_tasks,
                'timeElapsed': 0.0,
                'state': 'editable',
                'session_id': str(uuid.uuid4()),
            }

            r = make_response(json.dumps(label_header))
            r.mimetype = 'application/json'
            return r


        @app.route('/labelling/set_labels', methods=['POST'])
        def set_labels():
            label_header = json.loads(request.form['labels'])
            image_id = label_header['image_id']

            image = images_table[image_id]

            wrapped_labels = image.labels_store.get_wrapped_labels()
            wrapped_labels.labels_json = label_header['labels']
            wrapped_labels.completed_tasks = label_header['completed_tasks']
            image.labels_store.update_wrapped_labels(wrapped_labels)

            return make_response('')


        @app.route('/labelling/dextr', methods=['POST'])
        def dextr():
            dextr_js = json.loads(request.form['dextr'])
            if 'request' in dextr_js:
                dextr_request_js = dextr_js['request']
                image_id = dextr_request_js['image_id']
                dextr_id = dextr_request_js['dextr_id']
                dextr_points = dextr_request_js['dextr_points']

                image = images_table[image_id]
                regions_js = apply_dextr_js(image, dextr_points)

                dextr_labels = dict(image_id=image_id, dextr_id=dextr_id, regions=regions_js)
                dextr_reply = dict(labels=[dextr_labels])

                return make_response(json.dumps(dextr_reply))
            elif 'poll' in dextr_js:
                dextr_reply = dict(labels=[])
                return make_response(json.dumps(dextr_reply))
            else:
                return make_response(json.dumps({'error': 'unknown_command'}))


    @app.route('/image/<image_id>')
    def get_image(image_id):
        image_source = images_table[image_id].image_source
        local_path = image_source.local_path
        if local_path is not None:
            return send_file(str(local_path))
        else:
            bin_image, mimetype = image_source.image_binary_and_mime_type()
            r = make_response(bin_image)
            r.mimetype = mimetype
            return r



    if socketio is not None:
        socketio.run(app, debug=debug, port=port, use_reloader=use_reloader)
    else:
        app.run(debug=debug, port=port, use_reloader=use_reloader)



@click.command()
@click.option('--images_dir', type=click.Path(dir_okay=True, file_okay=False, exists=True), default='./images')
@click.option('--images_pat', type=str, default='*.png|*.jpg')
@click.option('--labels_dir', type=click.Path(dir_okay=True, file_okay=False, writable=True))
@click.option('--readonly', is_flag=True, default=False, help='Don\'t persist changes to disk')
@click.option('--update_label_object_ids', is_flag=True, default=False, help='Update object IDs in label JSON files')
@click.option('--enable_dextr', is_flag=True, default=False)
@click.option('--dextr_weights', type=click.Path())
def run_app(images_dir, images_pat, labels_dir, readonly, update_label_object_ids,
            enable_dextr, dextr_weights):
    import pathlib
    import json
    import uuid
    from image_labelling_tool import labelled_image, labelling_tool

    if enable_dextr or dextr_weights is not None:
        from dextr.model import DextrModel
        import torch

        device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")

        if dextr_weights is not None:
            dextr_weights = pathlib.Path(dextr_weights).expanduser()
            dextr_model = torch.load(dextr_weights, map_location=device)
        else:
            dextr_model = DextrModel.pascalvoc_resunet101().to(device)

        dextr_model.eval()

        dextr_fn = lambda image, points: dextr_model.predict([image], points[None, :, :])[0] >= 0.5
    else:
        dextr_fn = None


    # Colour schemes
    # The user may select different colour schemes for different tasks.
    # If you have a lot of classes, it will be difficult to select colours that are easily distinguished
    # from one another. For one task e.g. segmentation, design a colour scheme that highlights the different
    # classes for that task, while another task e.g. fine-grained classification would use another scheme.
    # Each colour scheme is a dictionary containing the following:
    #   name: symbolic name (Python identifier)
    #   human_name: human readable name for UI
    # These colour schemes are going to split the classes by 'default' (all), natural, and artificial.
    # Not really useful, but demonstrates the feature.
    colour_schemes = [
        dict(name='default', human_name='All'),
        dict(name='natural', human_name='Natural'),
        dict(name='artificial', human_name='Artifical')
    ]

    # Specify our label classes, organised in groups.
    # `LabelClass` parameters are:
    #   symbolic name (Python identifier)
    #   human readable name for UI
    #   and colours by colour scheme, as a dict mapping colour scheme name to RGB value as a list
    # The label classes are arranged in groups and will be displayed as such in the UI.
    # `LabelClassGroup` parameters are:
    #   human readable name for UI
    #   label class (`LabelClass` instance) list
    label_classes = [
        labelling_tool.LabelClassGroup('Natural', [
            labelling_tool.LabelClass('tree', 'Trees', dict(default=[0, 255, 192], natural=[0, 255, 192],
                                                            artificial=[128, 128, 128])),
            labelling_tool.LabelClass('lake', 'Lake', dict(default=[0, 128, 255], natural=[0, 128, 255],
                                                           artificial=[128, 128, 128])),
            labelling_tool.LabelClass('flower', 'Flower', dict(default=[255, 96, 192], natural=[255, 192, 96],
                                                               artificial=[128, 128, 128])),
            labelling_tool.LabelClass('leaf', 'Leaf', dict(default=[65, 255, 0], natural=[65, 255, 0],
                                                           artificial=[128, 128, 128])),
            labelling_tool.LabelClass('stem', 'Stem', dict(default=[128, 64, 0], natural=[128, 64, 0],
                                                           artificial=[128, 128, 128])),
        ]),
        labelling_tool.LabelClassGroup('Artificial', [
            labelling_tool.LabelClass('building', 'Buildings', dict(default=[255, 128, 0], natural=[128, 128, 128],
                                                                   artificial=[255, 128, 0])),
            labelling_tool.LabelClass('wall', 'Wall', dict(default=[0, 128, 255], natural=[128, 128, 128],
                                                           artificial=[0, 128, 255])),
        ])]

    # Annotation controls
    # Labels may also have optional meta-data associated with them
    # You could use this for e.g. indicating if an object is fully visible, mostly visible or significantly obscured.
    # You could also indicate quality (e.g. blurriness, etc)
    # There are four types of annotation. They have some common properties:
    #   - name: symbolic name (Python identifier)
    #   - label_text: label text in UI
    #   Check boxes, radio buttons and popup menus also have:
    #     - visibility_label_text: [optional] if provided, label visibility can be filtered by this annotation value,
    #       in which case a drop down will appear in the UI allowing the user to select a filter value
    #       that will hide/show labels accordingly
    # Control types:
    # Check box (boolean value):
    #   `labelling_tool.AnnoControlCheckbox`; only the 3 common parameters listed above
    # Radio button (choice from a list):
    #   `labelling_tool.AnnoControlRadioButtons`; the 3 common parameters listed above and:
    #       - choices: list of `labelling_tool.AnnoControlRadioButtons.choice` that provide:
    #           - value: symbolic value name for choice
    #           - tooltip: extra information for user
    #       - label_on_own_line [optional]: if True, place the label and the buttons on a separate line in the UI
    # Popup menu (choice from a grouped list):
    #   `labelling_tool.AnnoControlPopupMenu`; the 3 common parameters listed above and:
    #       - groups: list of groups `labelling_tool.AnnoControlPopupMenu.group`:
    #           - label_text: label text in UI
    #           - choices: list of `labelling_tool.AnnoControlPopupMenu.choice` that provide:
    #               - value: symbolic value name for choice
    #               - label_text: choice label text in UI
    #               - tooltip: extra information for user
    # Text (free form plain text):
    #   `labelling_tool.AnnoControlText`; only the 2 common parameters listed above and:
    #       - multiline: boolean; if True a text area will be used, if False a single line text entry
    anno_controls = [
        labelling_tool.AnnoControlCheckbox('good_quality', 'Good quality',
                                           visibility_label_text='Filter by good quality'),
        labelling_tool.AnnoControlRadioButtons('visibility', 'Visible', choices=[
            labelling_tool.AnnoControlRadioButtons.choice(value='full', label_text='Fully',
                                                          tooltip='Object is fully visible'),
            labelling_tool.AnnoControlRadioButtons.choice(value='mostly', label_text='Mostly',
                                                          tooltip='Object is mostly visible'),
            labelling_tool.AnnoControlRadioButtons.choice(value='obscured', label_text='Obscured',
                                                          tooltip='Object is significantly obscured'),
        ], label_on_own_line=False, visibility_label_text='Filter by visibility'),
        labelling_tool.AnnoControlPopupMenu('material', 'Material', groups=[
            labelling_tool.AnnoControlPopupMenu.group(label_text='Artifical/buildings', choices=[
                labelling_tool.AnnoControlPopupMenu.choice(value='concrete', label_text='Concrete',
                                                           tooltip='Concrete objects'),
                labelling_tool.AnnoControlPopupMenu.choice(value='plastic', label_text='Plastic',
                                                           tooltip='Plastic objects'),
                labelling_tool.AnnoControlPopupMenu.choice(value='asphalt', label_text='Asphalt',
                                                           tooltip='Road, pavement, etc.'),
            ]),
            labelling_tool.AnnoControlPopupMenu.group(label_text='Flat natural', choices=[
                labelling_tool.AnnoControlPopupMenu.choice(value='grass', label_text='Grass',
                                                           tooltip='Grass covered ground'),
                labelling_tool.AnnoControlPopupMenu.choice(value='water', label_text='Water', tooltip='Water/lake')]),
            labelling_tool.AnnoControlPopupMenu.group(label_text='Vegetation', choices=[
                labelling_tool.AnnoControlPopupMenu.choice(value='trees', label_text='Trees', tooltip='Trees'),
                labelling_tool.AnnoControlPopupMenu.choice(value='shrubbery', label_text='Shrubs',
                                                           tooltip='Shrubs/bushes'),
                labelling_tool.AnnoControlPopupMenu.choice(value='flowers', label_text='Flowers',
                                                           tooltip='Flowers'),
                labelling_tool.AnnoControlPopupMenu.choice(value='ivy', label_text='Ivy', tooltip='Ivy')]),
        ], visibility_label_text='Filter by material'),
        # labelling_tool.AnnoControlText('comment', 'Comment', multiline=False),
    ]

    image_pats = images_pat.split('|')

    # Load in .JPG images from the 'images' directory.
    labelled_images = labelled_image.LabelledImage.for_directory(
        images_dir, image_filename_patterns=image_pats, readonly=readonly)
    print('Loaded {0} images'.format(len(labelled_images)))

    if update_label_object_ids:
        n_updated = 0
        for limg in labelled_images:
            if limg.labels_store.labels_path.exists():
                label_js = json.load(limg.labels_store.labels_path.open('r'))
                prefix = str(uuid.uuid4())
                modified = labelling_tool.ensure_json_object_ids_have_prefix(
                    label_js, id_prefix=prefix)
                if modified:
                    with open(limg.labels_store.labels_path, 'w') as f_out:
                        json.dump(label_js, f_out, indent=3)
                    n_updated += 1
        print('Updated object IDs in {} files'.format(n_updated))



    # For documentation of the configuration, please see the comment above `labelling_tool.DEFAULT_CONFIG`
    config = labelling_tool.DEFAULT_CONFIG

    tasks = [
        dict(name='finished', human_name='[old] finished'),
        dict(name='segmentation', human_name='Outlines'),
        dict(name='classification', human_name='Classification'),
    ]

    flask_labeller(label_classes, labelled_images, tasks=tasks, colour_schemes=colour_schemes,
                   anno_controls=anno_controls, config=config, dextr_fn=dextr_fn)


if __name__ == '__main__':
    run_app()


# The MIT License (MIT)
#
# Copyright (c) 2015 University of East Anglia, Norwich, UK
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# Developed by Geoffrey French in collaboration with Dr. M. Fisher and
# Dr. M. Mackiewicz.
import os
import atexit
import socket
from copy import deepcopy
import json
import pathlib
import collections
import multiprocessing, queue
import urllib.parse
from image_labelling_tool import labelling_tool

def is_port_in_use(port):
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        return s.connect_ex(('localhost', port)) == 0


DEFAULT_PORT = 5000

DEFAULT_CONFIG = deepcopy(labelling_tool.DEFAULT_CONFIG)
# No fullscreen button for Qt-based labellers
DEFAULT_CONFIG.setdefault('settings', dict()).setdefault('fullscreenButton', False)

# The two kinds of images that we share between the main process and the Flask web server
# process
# _ImagePath: read from a file on disk
_ImagePath = collections.namedtuple('_ImagePath', ['path', 'width', 'height'])
# _ImageBinary: contains the binary image data along with a MIME type
_ImageBinary = collections.namedtuple('_ImageBinary', ['data', 'mime_type', 'width', 'height'])


class LabellerServer:
    """
    Global image labellers registry for Qt based labellers

    Invoke the `start_flask_server` method to start the server that will serve the index page,
    static files (CSS/JS) and the images that are to be labelled.
    """
    def __init__(self, port=DEFAULT_PORT):
        while is_port_in_use(port):
            port += 1
        self.img_reg_command_q = multiprocessing.Queue()
        self.server_process = None
        self.port = port

    def start_flask_server(self):
        if self.server_process is None:
            img_reg = _ImageRegistryConsumer(self.img_reg_command_q)
            self.server_process = multiprocessing.Process(
                target=_flask_server, args=(img_reg,), kwargs=dict(port=self.port))
            self.server_process.start()
            atexit.register(_kill_server_at_exit, self)
        return self.server_process

    def stop_server(self):
        if self.server_process is not None:
            self.server_process.terminate()
            self.server_process = None

    def server_url(self, tool_id=None, dextr_available=False):
        query_dict = {}
        if dextr_available:
            query_dict['dextr'] = ''
        if tool_id is not None:
            query_dict['tool_id'] = tool_id
        query = urllib.parse.urlencode(query_dict)
        return 'http://127.0.0.1:{}/?{}'.format(self.port, query)

    def image_registry(self):
        return _ImageRegistry(self.img_reg_command_q)


def _kill_server_at_exit(server):
    server.stop_server()


class _ImageRegistryConsumer:
    """
    The Flask server uses an image registry consumer to serve the images made
    available by each labeller.
    Labellers add images to an image registry, after which an image registry consumer
    allows the server to access them.
    """
    def __init__(self, command_q):
        self.command_q = command_q
        self.images = {}
        self.settings = {}

    def update(self):
        while True:
            try:
                command = self.command_q.get_nowait()
            except queue.Empty:
                break

            op = command['op']
            if op == 'add_image':
                image = command['image']
                image_id = command['image_id']
                self.images[image_id] = image
            elif op == 'remove_image':
                image_id = command['image_id']
                del self.images[image_id]
            elif op == 'add_settings':
                tool_id = command['tool_id']
                settings = command['settings']
                self.settings[tool_id] = settings
            elif op == 'remove_settings':
                tool_id = command['tool_id']
                del self.settings[tool_id]
            else:
                raise ValueError('Unknown operation {}'.format(op))

    def get_image(self, image_id):
        return self.images[image_id]

    def get_settings(self, tool_id):
        return self.settings[tool_id]


class _ImageRegistry:
    """
    The image registry provides `add_image` and `remove_image` methods for manipulating the list of images
    that are made available to the flask server.
    """
    def __init__(self, command_q):
        self.command_q = command_q

    def add_image(self, image_id, image):
        command = dict(op='add_image', image_id=image_id, image=image)
        self.command_q.put(command)

    def remove_image(self, image_id):
        command = dict(op='remove_image', image_id=image_id)
        self.command_q.put(command)

    def add_settings(self, tool_id, settings):
        command = dict(op='add_settings', tool_id=tool_id, settings=settings)
        self.command_q.put(command)

    def remove_settings(self, tool_id):
        command = dict(op='remove_settings', tool_id=tool_id)
        self.command_q.put(command)


def _flask_server(img_reg, port=5000, debug=False):
    """
    Internal helper function that runs the Flask server

    :param img_reg: an `_ImageRegistryConsumer` instance that provides images that the server will make available
    :param port: port for the server to listen on
    :param debug: if True, enable Flask debuggin
    """
    import json
    from flask import Flask, render_template, send_file, make_response, abort, request

    my_path = pathlib.Path(__file__)
    template_dir = my_path.parent.parent / 'image_labelling_tool' / 'templates'
    static_dir = my_path.parent.parent / 'image_labelling_tool' / 'static'

    app = Flask(__name__, static_folder=str(static_dir), template_folder=str(template_dir))

    @app.route('/')
    def index():
        settings = None
        tool_id = request.args.get('tool_id')
        if tool_id is not None:
            img_reg.update()
            settings = img_reg.get_settings(tool_id)

        if settings is not None:
            config = settings.get('config', DEFAULT_CONFIG)
            tasks = settings.get('tasks', None)
            colour_schemes = settings.get('colour_schemes', None)
            label_class_groups = settings.get('label_class_groups', [])
            anno_controls = settings.get('anno_controls', [])
            enable_firebug = settings.get('enable_firebug', False)
        else:
            config = DEFAULT_CONFIG
            tasks = None
            colour_schemes = None
            label_class_groups = []
            anno_controls = []
            enable_firebug = False

        dextr_available = 'dextr' in request.args

        return render_template('labeller_control_qt.jinja2',
                               labelling_tool_config=config,
                               tasks=tasks,
                               colour_schemes=colour_schemes,
                               label_class_groups=label_class_groups,
                               anno_controls=anno_controls,
                               dextr_available=dextr_available,
                               enable_firebug=enable_firebug)

    @app.route('/image/<image_id>')
    def get_image(image_id):
        img_reg.update()
        try:
            image = img_reg.get_image(image_id)
        except KeyError:
            abort(404)

        if isinstance(image, _ImagePath):
            return send_file(image.path)
        elif isinstance(image, _ImageBinary):
            r = make_response(image.data)
            r.mimetype = image.mime_type
            return r
        else:
            raise TypeError('Unknown image type {}'.format(type(image)))

    app.run(debug=debug, port=port, use_reloader=False)



# Generated by Django 3.0.3 on 2020-02-23 10:09

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('image_labelling_tool', '0003_auto_20180911_0954'),
    ]

    operations = [
        migrations.AlterField(
            model_name='labels',
            name='last_modified_by',
            field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='modified_labels', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='labels',
            name='locked_by',
            field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='locked_labels', to=settings.AUTH_USER_MODEL),
        ),
    ]



import json, datetime, uuid

from django.http import HttpResponse, JsonResponse
from django.views.decorators.cache import never_cache
from django.views import View
from django.utils.decorators import method_decorator

from django.conf import settings

from . import models


class LabellingToolView (View):
    """
    Labelling tool class based view

    Subclass and override the `get_labels` method (mandatory) and optionally
    the `update_labels` method to customise how label data is accessed and updated.

    `get_labels` should return a `models.Labels` instance or a dict of the form:
        {
            'completed_tasks': <list of names of completed tasks>
            'labels': labels as JSON data
            'state': 'locked'|'editable' to either disable/enable editing (e.g. if another user is editing them)
        }

    The `get_labels_for_update` method is the same, but is called by `update_labels`
    to retrieve a `Labels` instance to be updated. This could be used in a scenario in which
    you want to have `get_labels` return automatically generated labels when viewing,
    but apply updates to different manually created labels.

    Example:
    >>> class MyLabelView (LabellingToolView):
    ...     def get_labels(self, request, image_id_str, *args, **kwargs):
    ...         image = models.Image.get(id=int(image_id_string))
    ...         # Assume `image.labels` is a field that refers to the `Labels` instance
    ...         return image.labels

    Or:
    >>> class MyLabelView (LabellingToolView):
    ...     def get_labels(self, request, image_id_str, *args, **kwargs):
    ...         image = models.Image.get(id=int(image_id_string))
    ...         # Lets assume that the label data has been incorporated into the `Image` class:
    ...         labels_metadata = {
    ...             'completed_tasks': [task.name for task in image.completed_tasks.all()],
    ...             'labels': image.labels_json,
    ...             'state': ('locked' if image.in_use else 'editable')
    ...         }
    ...         return labels_metadata
    ...
    ...     def update_labels(self, request, image_id_str, labels, completed_tasks, time_elapsed, *args, **kwargs):
    ...         image = models.Image.get(id=int(image_id_string))
    ...         image.completed_tasks.set(completed_tasks)
    ...         image.edit_time_elapsed = time_elapsed
    ...         image.labels_json = labels
    ...         image.save()
    """
    def get_labels(self, request, image_id_str, *args, **kwargs):
        """
        Retrieve the `Labels` instance identified by `image_id_str` for display

        :param request: HTTP request
        :param image_id_str: image ID that identifies the image that we are labelling
        :param args: additional arguments
        :param kwargs:additional keyword arguments
        :return: a `Labels` instance
        """
        raise NotImplementedError('Abstract for type {}'.format(type(self)))

    def get_labels_for_update(self, request, image_id_str, *args, **kwargs):
        """
        Retrieve the `Labels` instance identified by `image_id_str` for updating

        :param request: HTTP request
        :param image_id_str: image ID that identifies the image that we are labelling
        :param args: additional arguments
        :param kwargs:additional keyword arguments
        :return: a `Labels` instance
        """
        return self.get_labels(request, image_id_str, *args, **kwargs)

    def update_labels(self, request, image_id_str, labels_json, completed_tasks, time_elapsed, *args, **kwargs):
        """
        Update the `Labels` instance identified by `image_id_str`

        :param request: HTTP request
        :param image_id_str: image ID that identifies the image that we are labelling
        :param labels_json: labels in JSON format (Python objects, not as a string)
        :param completed_tasks: sequence of `LabellingTask` instances that lists the tasks that have been completed
        :param time_elapsed: the amount of time taken by users to label this image
        :param args: additional arguments
        :param kwargs:additional keyword arguments
        :return: the `Labels` instance that was updated
        """
        labels = self.get_labels_for_update(request, image_id_str, *args, **kwargs)
        labels.update_labels(labels_json, completed_tasks, time_elapsed, request.user, save=True, check_lock=False)
        return labels

    def dextr_request(self, request, image_id_str, dextr_id, dextr_points):
        """
        :param request: HTTP request
        :param image_id_str: image ID that identifies the image that we are labelling
        :param dextr_id: an ID number the identifies the DEXTR request
        :param dextr_points: the 4 points as a list of 2D vectors ({'x': <x>, 'y': <y>}) in the order
            top edge, left edge, bottom edge, right edge
        :return: contours/regions a list of lists of 2D vectors, each of which is {'x': <x>, 'y': <y>}
        """
        raise NotImplementedError('abstract: dextr_request not implemented for {}'.format(type(self)))

    def dextr_poll(self, request, image_id_str, dextr_ids):
        """
        :param request: HTTP request
        :param image_id_str: image ID that identifies the image that we are labelling
        :param dextr_ids: The DEXTR request IDs that the client is interested in
        :return: a list of dicts where each dict takes the form:
            {
                'image_id': image ID string that identifies the image that the label applies to
                'dextr_id': the ID number that identifies the dextr job/request
                'regions': contours/regions a list of lists of 2D vectors, each of which is {'x': <x>, 'y': <y>}
            }
        """
        raise NotImplementedError('abstract: dextr_poll not implemented for {}'.format(type(self)))

    @method_decorator(never_cache)
    def get(self, request, *args, **kwargs):
        if 'labels_for_image_id' in request.GET:
            image_id_str = request.GET['labels_for_image_id']

            session_id = str(uuid.uuid4())

            labels = self.get_labels(request, image_id_str, *args, **kwargs)
            if labels is None:
                # No labels for this image
                labels_header = {
                    'image_id': image_id_str,
                    'completed_tasks': [],
                    'timeElapsed': 0.0,
                    'state': 'editable',
                    'labels': [],
                    'session_id': session_id,
                }
            elif isinstance(labels, models.Labels):
                # Remove existing lock
                labels_header = {
                    'image_id': image_id_str,
                    'completed_tasks': [task.name for task in labels.completed_tasks.all()],
                    'timeElapsed': labels.edit_time_elapsed,
                    'state': 'editable',
                    'labels': labels.labels_json,
                    'session_id': session_id,
                }
            elif isinstance(labels, dict):
                labels_header = {
                    'image_id': image_id_str,
                    'completed_tasks': labels['completed_tasks'],
                    'timeElapsed': labels.get('edit_time_elapsed', 0.0),
                    'state': labels.get('state', 'editable'),
                    'labels': labels['labels'],
                    'session_id': session_id,
                }
            else:
                raise TypeError('labels returned by get_labels metod should be None, a Labels model '
                                'or a dictionary; not a {}'.format(type(labels)))

            return JsonResponse(labels_header)
        elif 'next_unlocked_image_id_after' in request.GET:
            return JsonResponse({'error': 'operation_not_supported'})
        else:
            return JsonResponse({'error': 'unknown_operation'})

    @method_decorator(never_cache)
    def post(self, request, *args, **kwargs):
        if 'labels' in request.POST:
            # Write labels
            labels = json.loads(request.POST['labels'])
            image_id = labels['image_id']
            completed_task_names = labels['completed_tasks']
            time_elapsed = labels['timeElapsed']
            label_data = labels['labels']

            completed_tasks = models.LabellingTask.objects.filter(enabled=True, name__in=completed_task_names)

            try:
                self.update_labels(request, str(image_id), label_data, completed_tasks, time_elapsed, *args, **kwargs)
            except models.LabelsLockedError:
                return JsonResponse({'error': 'locked'})
            else:
                return JsonResponse({'response': 'success'})
        elif 'dextr' in request.POST:
            # DEXTR
            dextr_js = json.loads(request.POST['dextr'])
            if 'request' in dextr_js:
                dextr_request_js = dextr_js['request']
                image_id = dextr_request_js['image_id']
                dextr_id = dextr_request_js['dextr_id']
                dextr_points = dextr_request_js['dextr_points']

                regions_js = self.dextr_request(request, str(image_id), dextr_id, dextr_points)

                if regions_js is not None:
                    dextr_labels = dict(image_id=image_id, dextr_id=dextr_id, regions=regions_js)
                    dextr_reply = dict(labels=[dextr_labels])
                    return JsonResponse(dextr_reply)
                else:
                    return JsonResponse({'response': 'success'})
            elif 'poll' in dextr_js:
                dextr_poll_js = dextr_js['poll']
                image_id_str = dextr_poll_js['image_id']
                dextr_ids = dextr_poll_js['dextr_ids']

                dextr_ids = [int(x) for x in dextr_ids]

                labels_js = self.dextr_poll(request, image_id_str, dextr_ids)

                if labels_js is not None:
                    dextr_reply = dict(labels=labels_js)
                    return JsonResponse(dextr_reply)
                else:
                    return JsonResponse({'response': 'success'})
            if isinstance(dextr_js, dict):
                return JsonResponse({'error': 'unknown_dextr_api', 'keys': list(dextr_js.keys())})
            else:
                return JsonResponse({'error': 'unknown_dextr_api', 'type': str(type(dextr_js))})
        else:
            return JsonResponse({'response': 'unknown_api', 'keys': [str(k) for k in request.POST.keys()]})


class LabellingToolViewWithLocking (LabellingToolView):
    """
    Labelling tool class based view with label locking

    Subclass and override the `get_labels` method (mandatory), the
    `get_next_unlocked_image_id_after` method (mandatory) and optionally
    the `update_labels` method to customise how label data is accessed and updated.

    `get_labels` should return a `models.Labels` instance; it should NOT return anything else
    in the way that the `get_labels` method of a subclass of `LabellingToolView` can.

    The `LABELLING_TOOL_LOCK_TIME` attribute in settings can be used to set the amount of time
    that a lock lasts for in seconds; default is 10 minutes (600s).

    Example:
    >>> class MyLabelView (LabellingToolViewWithLocking):
    ...     def get_labels(self, request, image_id_str, *args, **kwargs):
    ...         image = models.Image.get(id=int(image_id_string))
    ...         # Assume `image.labels` is a field that refers to the `Labels` instance
    ...         return image.labels
    ...
    ...     def get_unlocked_image_id(self, request, image_ids, *args, **kwargs):
    ...         unlocked_labels = image_labelling_tool.models.Labels.objects.unlocked()
    ...         unlocked_q = Q(id__in=image_ids, labels__in=unlocked_labels)
    ...         # Optional: filter images for those accessible to the user to guard against maliciously crafted
    ...         # requests
    ...         accessible_q = Q(owner=request.user)
    ...         unlocked_imgs = models.Image.objects.filter(unlocked_q & accessible_q).distinct()
    ...         first_unlocked = unlocked_imgs.first()
    ...         return first_unlocked.id if first_unlocked is not None else None
    """
    def get_unlocked_image_id(self, request, image_ids, *args, **kwargs):
        """
        Get the ID of the first image from the list of provided image IDs that is unlocked.
        Images are locked when a user is editing them. This finds an image that is not being edited
        by someone else.

        :param request: HTTP request
        :param image_ids: list of image IDs to search
        :param args: additional arguments
        :param kwargs:additional keyword arguments
        :return: the ID of the next unlocked image, or `None` if not found
        """
        raise NotImplementedError('Abstract for type {}'.format(type(self)))

    def update_labels(self, request, image_id_str, labels_json, completed_tasks, time_elapsed, *args, **kwargs):
        """
        Update the `Labels` instance identified by `image_id_str`

        :param request: HTTP request
        :param image_id_str: image ID that identifies the image that we are labelling
        :param labels_json: labels in JSON format (Python objects, not as a string)
        :param completed_tasks: sequence of `LabellingTask` instances that lists the tasks that have been completed
        :param time_elapsed: the amount of time taken by users to label this image
        :param args: additional arguments
        :param kwargs:additional keyword arguments
        :return: the `Labels` instance that was updated
        """
        expire_after = getattr(settings, 'LABELLING_TOOL_LOCK_TIME', 600)
        labels = self.get_labels_for_update(request, image_id_str, *args, **kwargs)
        labels.update_labels(labels_json, completed_tasks, time_elapsed, request.user, check_lock=True, save=False)
        if request.user.is_authenticated:
            labels.refresh_lock(request.user, datetime.timedelta(seconds=expire_after), save=False)
        labels.save()
        return labels

    @method_decorator(never_cache)
    def get(self, request, *args, **kwargs):
        if 'labels_for_image_id' in request.GET:
            image_id_str = request.GET['labels_for_image_id']

            session_id = str(uuid.uuid4())

            labels = self.get_labels(request, image_id_str)

            if not isinstance(labels, models.Labels):
                raise TypeError('labels returned by get_labels metod should be a Labels '
                                'model, not a {}'.format(type(labels)))

            # Remove existing lock
            if request.user.is_authenticated:
                already_locked = models.Labels.objects.locked_by_user(request.user)
                for locked_labels in already_locked:
                    locked_labels.unlock(from_user=request.user, save=True)

            if labels.is_locked_to(request.user):
                state = 'locked'
                attempt_lock = False
            else:
                state = 'editable'
                attempt_lock = True
            labels_header = {
                'image_id': image_id_str,
                'completed_tasks': [task.name for task in labels.completed_tasks.all()],
                'timeElapsed': labels.edit_time_elapsed,
                'state': state,
                'labels': labels.labels_json,
                'session_id': session_id,
            }

            if attempt_lock and request.user.is_authenticated:
                expire_after = getattr(settings, 'LABELLING_TOOL_LOCK_TIME', 600)
                labels.lock(request.user, datetime.timedelta(seconds=expire_after), save=True)

            return JsonResponse(labels_header)
        else:
            return JsonResponse({'error': 'unknown_operation'})

    @method_decorator(never_cache)
    def post(self, request, *args, **kwargs):
        if 'get_unlocked_image_id' in request.POST:
            request_str = request.POST['get_unlocked_image_id']
            request = json.loads(request_str)
            image_ids = request['image_ids']
            unlocked_image_id = self.get_unlocked_image_id(request, image_ids)
            return JsonResponse({'image_id': str(unlocked_image_id)})
        else:
            return super(LabellingToolViewWithLocking, self).post(request, *args, **kwargs)




import json, datetime
from django.db import models
from django.conf import settings
from django.utils import timezone
from django.contrib.auth import get_user_model
from . import managers


class LabelsLockedError (Exception):
    pass


class LabellingTask (models.Model):
    enabled = models.BooleanField(default=True)
    name = models.CharField(max_length=256)
    human_name = models.CharField(max_length=256)
    order_key = models.IntegerField(default=0)

    def to_json(self):
        return dict(name=self.name, human_name=self.human_name)

    def __str__(self):
        return 'Task {} (identifier {})'.format(self.human_name, self.name)


class Labels (models.Model):
    # Label data
    labels_json_str = models.TextField(default='[]')

    # Task completion
    completed_tasks = models.ManyToManyField(LabellingTask)

    # Creation date
    creation_date = models.DateField()

    # Time elapsed during editing, in seconds
    edit_time_elapsed = models.FloatField(default=0.0, blank=True)

    # Last modification user and datetime
    last_modified_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, models.SET_NULL, related_name='modified_labels', null=True, default=None)
    last_modified_datetime = models.DateTimeField(default=datetime.datetime.now)

    # Locked by user and expiry datetime
    locked_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, models.SET_NULL, related_name='locked_labels', null=True, default=None)
    lock_expiry_datetime = models.DateTimeField(default=datetime.datetime.now)

    # Manager
    objects = managers.LabelsManager()

    @property
    def labels_json(self):
        return json.loads(self.labels_json_str)

    @labels_json.setter
    def labels_json(self, label_js):
        self.labels_json_str = json.dumps(label_js)

    @property
    def metadata(self):
        """
        Access metadata (completed tasks, creation date, last modified by, last modified date time) as a dict
        :return:
        """
        return dict(
            completed_tasks=list(self.completed_tasks.all()),
            creation_date=self.creation_date,
            last_modified_by=self.last_modified_by,
            last_modified_datetime=self.last_modified_datetime
        )

    @metadata.setter
    def metadata(self, meta):
        if 'completed_tasks' in meta:
            self.completed_tasks.set(meta['completed_tasks'])
        if 'creation_date' in meta:
            self.creation_date = meta['creation_date']
        if 'last_modified_by' in meta:
            self.last_modified_by = meta['last_modified_by']
        if 'last_modified_datetime' in meta:
            self.last_modified_datetime = meta['last_modified_datetime']

    @property
    def metadata_json(self):
        """
        Access metadata (completed tasks, creation date, last modified by, last modified date time) as a
        JSON dict. The 'last modified by' user is stored as user name and/or user ID. The completed tasks
        are stored by name. Dates and datetimes are stored in string form.
        :return:
        """
        return self.metadata_dict_to_json(self.metadata)

    @metadata_json.setter
    def metadata_json(self, meta_js):
        self.metadata = self.metadata_json_to_dict(meta_js)


    @staticmethod
    def metadata_dict_to_json(metadata):
        """
        Convert metadata in dictionary form to JSON form.

        Dates and date times are converted to string form for storage as JSON.
        The last_modified_by User object is stored in JSON as a username and user ID.
        The completed tasks are converted to a list of task names

        :param metadata: metadata in a dictionary with the following optional keys: 'creation_date', 'last_modified_by',
            'last_modified_datetime' and 'completed_tasks'
        :return: metadata in JSON form
        """
        meta_json = {}

        if 'creation_date' in metadata:
            meta_json['creation_date'] = metadata['creation_date'].strftime('%Y-%m-%d')

        if 'last_modified_by' in metadata:
            last_modified_by = metadata['last_modified_by']
            if last_modified_by is not None:
                meta_json['last_modified_by__username'] = last_modified_by.username
                meta_json['last_modified_by__id'] = last_modified_by.id

        if 'last_modified_datetime' in metadata:
            meta_json['last_modified_datetime'] = metadata['last_modified_datetime'].strftime('%Y-%m-%d %H:%M:%S')

        if 'completed_tasks' in metadata:
            meta_json['completed_tasks'] = [task.name for task in metadata['completed_tasks']]

        return meta_json

    @staticmethod
    def metadata_json_to_dict(metadata_json):
        """
        Convert metadata as a JSON dictionary to dictionary form.

        :param metadata_json: metadata as a JSON dictionary
        :return: metadata in dict form
        """
        meta = {}

        if 'creation_date' in metadata_json:
            meta['creation_date'] = datetime.datetime.strptime(metadata_json['creation_date'], '%Y-%m-%d').date()

        last_modified_by = None
        if 'last_modified_by__username' in metadata_json:
            username = metadata_json['last_modified_by__username']
            last_modified_by = get_user_model().objects.get(username=username)
        if last_modified_by is None and 'last_modified_by__id' in metadata_json:
            user_id = metadata_json['last_modified_by__id']
            last_modified_by = get_user_model().objects.get(id=user_id)
        if last_modified_by is not None:
            meta['last_modified_by'] = last_modified_by

        if 'last_modified_datetime' in metadata_json:
            meta['last_modified_datetime'] = datetime.datetime.strptime(metadata_json['last_modified_datetime'],
                                                                        '%Y-%m-%d %H:%M:%S')

        if 'complete' in metadata_json:
            completed_task_names = ['finished']
        elif 'completed_tasks' in metadata_json:
            completed_task_names = metadata_json['completed_tasks']
        else:
            completed_task_names = None
        if completed_task_names is not None:
            meta['completed_tasks'] = list(LabellingTask.objects.filter(name__in=completed_task_names).distinct())

        return meta


    @staticmethod
    def from_labels_json_str_and_metadata_dict(labels_json_str, metadata):
        keys = ['creation_date', 'completed_tasks', 'last_modified_by', 'last_modified_datetime']
        kwargs = {key: metadata[key] for key in keys}

        return Labels(labels_json_str=labels_json_str, **kwargs)

    @staticmethod
    def from_labels_json_str_and_metadata_json(labels_json_str, metadata_json):
        return Labels.from_labels_json_str_and_metadata_dict(
            labels_json_str, Labels.metadata_json_to_dict(metadata_json))


    @property
    def is_empty(self):
        return self.labels_json_str == '[]'

    @property
    def label_classes(self):
        label_classes = [x['label_class']   for x in self.labels_json]
        return set(label_classes)

    @property
    def label_class_histogram(self):
        if self.is_empty:
            return {}
        else:
            histogram = {}
            for x in self.labels_json:
                cls = x['label_class']
                histogram[cls] = histogram.get(cls, 0) + 1
            return histogram

    def update_labels(self, labels_json, completed_tasks, time_elapsed, user, save=False, check_lock=False):
        """
        Update labels, normally called by Django views that are responding to user input received from the client

        :param labels_json: labels in JSON form
        :param completed_tasks: sequence of LabellingTask instances
        :param time_elapsed: labelling time elapsed
        :param user: user account being used to edit the labels
        :param save: if `True`, invoke `self.save()` afterwards
        :param check_lock: if `True`, raise `LabelsLockedError` if this labels instance is locked by another user
        """
        # Verify time elapsed is within the bounds of possibility
        current_time = timezone.now()
        dt_since_last_mod = (current_time - self.last_modified_datetime).total_seconds()
        # Allow to either double the time since last modification or time since last modification plus 1 minute
        # to account for potential latency in delivery of last edit
        permitted_dt = max(dt_since_last_mod * 2.0, dt_since_last_mod + 60.0)
        permitted_time = self.edit_time_elapsed + permitted_dt
        if time_elapsed > permitted_time:
            print('WARNING: rejecting time_elapsed: '
                  'self.edit_time_elapsed={}, time_elapsed={}, permitted_time={}'.format(
                        self.edit_time_elapsed, time_elapsed, permitted_time
            ))
        elif time_elapsed >= self.edit_time_elapsed:
            self.edit_time_elapsed = time_elapsed

        if check_lock:
            if self.is_locked_to(user):
                raise LabelsLockedError
        self.labels_json = labels_json
        self.completed_tasks.set(completed_tasks)
        if user.is_authenticated:
            self.last_modified_by = user
        else:
            self.last_modified_by = None
        self.last_modified_datetime = timezone.now()
        if save:
            self.save()

    def is_lock_active(self):
        return timezone.now() < self.lock_expiry_datetime and self.locked_by is not None

    def is_locked_to(self, user=None):
        lock_active = self.is_lock_active()
        if user is not None and not user.is_authenticated:
            user = None
        if user is None:
            return lock_active
        else:
            return lock_active and user != self.locked_by

    def lock(self, to_user, expire_after, save=False):
        if self.is_locked_to(to_user):
            raise ValueError('Cannot lock Labels(id={}) to user {}; is already locked'.format(
                self.id, to_user.username
            ))
        self.locked_by = to_user
        expiry = timezone.now() + expire_after
        self.lock_expiry_datetime = expiry
        if save:
            self.save()

    def refresh_lock(self, to_user, expire_after, save=False):
        if self.is_lock_active():
            if self.locked_by != to_user:
                raise ValueError('Cannot refresh lock Labels(id={}) for user {}; is already locked by {}'.format(
                    self.id, to_user.username, self.locked_by.username
                ))
        expiry = timezone.now() + expire_after
        self.lock_expiry_datetime = expiry
        if save:
            self.save()

    def unlock(self, from_user, save=False):
        if self.is_lock_active():
            if from_user != self.locked_by:
                raise ValueError('Cannot unlock Labels(id={}) from user {}, it is locked by {}'.format(
                    self.id, from_user.username, self.locked_by.username
                ))
            self.locked_by = None
            self.lock_expiry_datetime = timezone.now()
            if save:
                self.save()

    def __str__(self):
        if self.last_modified_by is not None:
            return 'Labels {} (last modified by {} at {})'.format(
                self.id, self.last_modified_by.username, self.last_modified_datetime)
        else:
            return 'Labels {}'.format(self.id)



import uuid, json

from django import template
from django.utils.html import format_html

from image_labelling_tool import labelling_tool as lt
from image_labelling_tool import models as lt_models

register = template.Library()


@register.filter(name='as_json')
def as_json(value):
    return json.dumps(value)


def _update_config(dest, src):
    if isinstance(src, dict):
        for key, value in src.items():
            if isinstance(value, dict) and isinstance(dest.get(key), dict):
                print('Updating {}...'.format(key))
                _update_config(dest[key], src[key])
            else:
                dest[key] = value


@register.inclusion_tag('inline/image_labeller.html', name='labelling_tool')
def labelling_tool(label_class_groups, image_descriptors, color_schemes, initial_image_index,
                   labelling_tool_url, tasks=None, anno_controls=None, enable_locking=False, dextr_available=False, dextr_polling_interval=None,
                   config=None):
    if config is None:
        config = {}
    if dextr_polling_interval is not None:
        dextr_polling_interval = str(dextr_polling_interval)
    else:
        dextr_polling_interval = 'null'

    if anno_controls is None:
        anno_controls = []

    if tasks is None:
        tasks_json = [dict(identifier='finished', human_name='Finished')]
    else:
        tasks_json = []
        for task in tasks:
            if isinstance(task, dict):
                tasks_json.append(task)
            elif isinstance(task, lt_models.LabellingTask):
                tasks_json.append(task.to_json())
            else:
                raise TypeError('tasks should be an iterable of JSON dictionaries or '
                                'LabellingTask instances, not {}'.format(type(task)))

    return {
        'colour_schemes': color_schemes,
        'tasks': tasks_json,
        'label_class_groups': label_class_groups,
        'anno_controls': anno_controls,
        'image_descriptors': image_descriptors,
        'num_images': len(image_descriptors),
        'initial_image_index': str(initial_image_index),
        'labelling_tool_url': labelling_tool_url,
        'enable_locking': enable_locking,
        'dextr_available': dextr_available,
        'dextr_polling_interval': dextr_polling_interval,
        'labelling_tool_config': config,
    }



# Generated by Django 3.0.3 on 2020-08-25 10:28

from django.db import migrations, transaction, models


def complete_to_completed_tasks(apps, schema_editor):
    Labels = apps.get_model('image_labelling_tool', 'Labels')
    LabellingTask = apps.get_model('image_labelling_tool', 'LabellingTask')

    with transaction.atomic():
        if Labels.objects.filter(complete=True).exists():
            # There are some Label instances that are marked as finished
            # Create 'finished' task if it does not exist
            try:
                finished_task = LabellingTask.objects.get(name='finished')
            except LabellingTask.DoesNotExist:
                finished_task = LabellingTask(name='finished', human_name='(old) Finished', enabled=True)
                finished_task.save()

            rows = Labels.objects.all()
            for row in rows:
                if row.complete:
                    row.completed_tasks.add(finished_task)
                    row.save()


class Migration(migrations.Migration):
    dependencies = [
        ('image_labelling_tool', '0004_auto_20200223_1009'),
    ]

    operations = [
        migrations.CreateModel(
            name='LabellingTask',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enabled', models.BooleanField(default=True)),
                ('name', models.CharField(max_length=256)),
                ('human_name', models.CharField(max_length=256)),
                ('order_key', models.IntegerField(default=0)),
            ],
        ),
        migrations.AddField(
            model_name='labels',
            name='completed_tasks',
            field=models.ManyToManyField(to='image_labelling_tool.LabellingTask'),
        ),
        migrations.RunPython(complete_to_completed_tasks),
    ]



import os
import fnmatch
from setuptools import find_packages
from setuptools import setup

version = '0.2.3'

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.md')).read()
except IOError:
    README = ''

def find_data_files(dir, pat):
    files = []
    for f in os.listdir(dir):
        if fnmatch.fnmatch(f, pat):
            files.append(os.path.join(dir, f))
    return (dir, files)

install_requires = [
    'numpy',
    'Pillow',
    'scikit-image',
    'click',
    'flask',
    'deprecated'
]

tests_require = [
]

django_require = [
    'django'
]

dextr_require = [
    'dextr'
]

qt_require = [
    'PyQt5'
]

include_package_data = True
data_files = [
    ('image_labelling_tool/templates', [
        'image_labelling_tool/templates/labeller_page.jinja2'
        'image_labelling_tool/templates/labeller_control_qt.jinja2'
    ]),
    ('image_labelling_tool/templates/inline', [
        'image_labelling_tool/templates/inline/labeller_app.html',
        'image_labelling_tool/templates/inline/image_labeller.html',
        'image_labelling_tool/templates/inline/image_labeller_css.html',
        'image_labelling_tool/templates/inline/image_labeller_scripts.html',
    ]),
    find_data_files('image_labelling_tool/static', '*.*'),
    find_data_files('image_labelling_tool/static/open-iconic/css', '*.*'),
    find_data_files('image_labelling_tool/static/open-iconic/fonts', '*.*'),
    find_data_files('image_labelling_tool/static/labelling_tool', '*.*'),
]

setup(
    name="django-labeller",
    version=version,
    description="An image labelling tool for creating segmentation data sets, for Django, Flask and Qt.",
    long_description="\n\n".join([README]),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: User Interfaces",
    ],
    keywords="",
    author="Geoff French",
    # author_email="brittix1023 at gmail dot com",
    url="https://github.com/Britefury/django-labeller",
    license="MIT",
    packages=find_packages(),
    include_package_data=include_package_data,
    data_files=data_files,
    zip_safe=False,
    install_requires=install_requires,
    extras_require={
        'testing': tests_require,
        'django': django_require,
        'dextr': dextr_require,
        'qt5': qt_require,
    },
)
